from concurrent.futures import TimeoutError
from pebble import ProcessPool, ProcessExpired
import pandas as pd
import os
import sqlite3
import sys
import time
from tqdm import tqdm
    

def check_query(args):
    conn = sqlite3.connect('shopping.db',timeout=1.0)

    query, i, queryNumber, CORRECT_RESULTS = args
    message = ""
    
    try:
        start = time.process_time()
        answer = pd.read_sql_query(query, conn)
        end = time.process_time()
        processtime = end-start
        correct = CORRECT_RESULTS.get(queryNumber)
        score = 0 
        for item in correct:
            if answer.equals(item):
                score = 2
        return [score, message, processtime]
    except TypeError as e:
        #print(f"This was probably a testing query (undefined queryname {queryNumber})")
        return [1, message, 20]
    except Exception as e:
        #print(i, e)
        return [1, str(e), 20]

def calculateCorrectness(queryList, queryNumberList):
    pbar = tqdm(total=len(queryList))

    with ProcessPool() as pool:
        future = pool.map(check_query, [(query, i, queryNumberList[i], CORRECT_RESULTS) for i, query in enumerate(queryList)], timeout=TIMEOUT)

        iterator = future.result()
        result = []
        error = []
        processtime = []
        while True:
            try:
                item = next(iterator)
                pbar.update(1)
                result.append(item[0])
                error.append(item[1])
                processtime.append(item[2])
            except StopIteration:
                break
            except TimeoutError as e:
                #print("Query took longer than seconds", e.args)
                pbar.update(1)
                result.append(3)
                error.append("timeout")
                processtime.append(20)
            except Exception as e:
                print("Some other error: ", e.args)
                pbar.update(1)
                result.append(1)
                error.append(str(e))
                processtime.append(20)

    pbar.close()

    return result, error, processtime

if __name__ == '__main__':
    os.remove("shopping.db")
    conn = sqlite3.connect('shopping.db',timeout=1.0)
    cur = conn.cursor()

    db_location = 'shoppingDB.sql'

    with open(os.path.join(sys.path[0], db_location), "r") as f:
        sql = f.read()
        cur.executescript(sql)

    TIMEOUT = 5
    CORRECT_RESULTS = {
    'query1': #Select all different names and ids of the customers who never made a purchase at a store with the name ‘Kumar’. 
        # Also include the customers who never made a purchase, but are still represented in the customer relation of our database.
       [pd.read_sql_query('''SELECT pID FROM product 
            WHERE pID NOT IN (
                SELECT p1.pID 
                FROM purchase p1, purchase p2, purchase p3 
                WHERE p1.pID=p2.pID AND p1.pID=p3.pID 
                AND p1.cID<>p2.cID AND p1.cID<>p3.cID 
                AND p2.cID<>p3.cID
            )''', conn),
        pd.read_sql_query('''SELECT distinct pID FROM purchase 
            WHERE pID NOT IN (
                SELECT p1.pID 
                FROM purchase p1, purchase p2, purchase p3 
                WHERE p1.pID=p2.pID AND p1.pID=p3.pID 
                AND p1.cID<>p2.cID AND p1.cID<>p3.cID 
                AND p2.cID<>p3.cID
            )''', conn),
        pd.read_sql_query('''SELECT pID FROM purchase 
            WHERE pID NOT IN (
                SELECT p1.pID 
                FROM purchase p1, purchase p2, purchase p3 
                WHERE p1.pID=p2.pID AND p1.pID=p3.pID 
                AND p1.cID<>p2.cID AND p1.cID<>p3.cID 
                AND p2.cID<>p3.cID
            )''', conn)],
    'query2': #Select all different names and ids of customers who made at least one purchase at a store with the name ‘Kumar’ and never a purchase at a store with a different name than ‘Kumar’.        
        [pd.read_sql_query('''SELECT c.cID, c.cName 
            FROM customer c, purchase p1, purchase p2, product pr1, product pr2 
            WHERE c.cID=p1.cID AND p1.cID=p2.cID AND p1.date=p2.date 
                AND p1.pID=pr1.pID AND pr1.pName="Onions"
                AND p2.pID=pr2.pID AND pr2.pName="Coffee" ''', conn),
         pd.read_sql_query('''SELECT DISTINCT c.cID, c.cName 
            FROM customer c, purchase p1, purchase p2, product pr1, product pr2 
            WHERE c.cID=p1.cID AND p1.cID=p2.cID AND p1.date=p2.date 
                AND p1.pID=pr1.pID AND pr1.pName="Onions"
                AND p2.pID=pr2.pID AND pr2.pName="Coffee" ''', conn)
                ],
    'query3': #Select all distinct combinations of names and ids of customers who have both a shoppinglist and a purchase, both for the same date in 2018.
        [pd.read_sql_query('''WITH cust_no_purchase_at_store(cID, sID) AS ( 
                                SELECT c.cID, s.sID 
                                FROM customer c, store s 
                                EXCEPT 
                                SELECT p.cID, p.sID 
                                FROM purchase p 
                            ) 
                            SELECT cID 
                            FROM customer 
                            EXCEPT 
                            SELECT cID 
                            FROM cust_no_purchase_at_store''', conn)
                ],
    'query4': #Write a SQL query for the following RA query
        [pd.read_sql_query('''SELECT s.sName 
            FROM store s 
            GROUP BY s.sName 
            HAVING COUNT(s.sID) > 1 
              INTERSECT 
            SELECT s.sName 
            FROM purchase p, store s 
            WHERE p.sID=s.sID 
            GROUP BY s.sName 
            HAVING AVG(p.quantity) > 4''', conn)     
                ],
    'query5': #List the names of all of those customers, and only of those customers, 
        #that spent an amount of money on any one date that is at least 75% of the maximal amount of money ever spent by a single customer on a single day.
        [pd.read_sql_query('''SELECT pr.pName, s.sName 
            FROM product pr, purchase p, store s 
            WHERE p.pID=pr.pID AND p.sID=s.sID AND p.price IN ( 
                SELECT MAX(price) 
                FROM purchase p 
            )''', conn), 

        pd.read_sql_query('''SELECT DISTINCT pr.pName, s.sName 
            FROM product pr, purchase p, store s 
            WHERE p.pID=pr.pID AND p.sID=s.sID AND p.price IN ( 
                SELECT MAX(price) 
                FROM purchase p 
            )''', conn), 

        pd.read_sql_query('''SELECT pr.pName, s.sName 
            FROM product pr, purchase p, store s 
            WHERE p.pID=pr.pID AND p.sID=s.sID AND p.price/p.quantity IN ( 
                SELECT MAX(price/p.quantity) 
                FROM purchase p 
            )''', conn), 

        pd.read_sql_query('''SELECT DISTINCT pr.pName, s.sName 
            FROM product pr, purchase p, store s 
            WHERE p.pID=pr.pID AND p.sID=s.sID AND p.price/p.quantity IN ( 
                SELECT MAX(price/p.quantity) 
                FROM purchase p 
            )''', conn)

        
        
                ],
    'query6':
        [pd.read_sql_query('''SELECT MAX(i1.unit_price-i2.unit_price) 
            FROM inventory i1, inventory i2 
            WHERE i1.pID=i2.pID AND i1.sID<>i2.sID 
                AND i1.date=i2.date AND i1.date="2018-08-23" ''', conn), 

        pd.read_sql_query('''SELECT MIN(i1.unit_price-i2.unit_price) 
            FROM inventory i1, inventory i2 
            WHERE i1.pID=i2.pID AND i1.sID<>i2.sID 
                AND i1.date=i2.date AND i1.date="2018-08-23" ''', conn)
        ]
    }

    filename = "output.csv"
    data = pd.read_csv(filename, sep='─', header=0)

    correctness = calculateCorrectness(data['query'].tolist(), data['question'].tolist())
    data['correct'] = correctness[0]
    data['errormessage'] = correctness[1]
    data['runningtime'] = correctness[2]
    data.head(20)

    data.to_csv("output_correct.csv", sep='─', index=False, header=True)









